@@ -11,12 +11,16 @@ from tf_geometric.nn.conv.tagcn import tagcn
 from tf_geometric.nn.conv.graph_sage import mean_graph_sage, mean_pool_graph_sage, max_pool_graph_sage, gcn_graph_sage, lstm_graph_sage
 from tf_geometric.nn.conv.appnp import appnp
 from tf_geometric.nn.conv.gin import gin
+from tf_geometric.nn.conv.le_conv import le_conv
+
 
 from tf_geometric.nn.sampling.drop_edge import drop_edge
 
 from tf_geometric.nn.pool.common_pool import mean_pool, min_pool, max_pool, sum_pool
 from tf_geometric.nn.pool.topk_pool import topk_pool
-from tf_geometric.nn.pool.diff_pool import diff_pool
+from tf_geometric.nn.pool.diff_pool import diff_pool, diff_pool_coarsen
 from tf_geometric.nn.pool.set2set import set2set
-
-
+from tf_geometric.nn.pool.cluster_pool import cluster_pool
+from tf_geometric.nn.pool.sag_pool import sag_pool
+from tf_geometric.nn.pool.asap import asap
+from tf_geometric.nn.pool.sort_pool import sort_pool
